# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import List

from pydantic_xml import BaseXmlModel, element

from .locale import Locale
from .plist import Plist

log = logging.getLogger("locale")


class Locales(BaseXmlModel):
    locales: List[Locale] = element(tag="locale")

    @property
    def main_locale(self) -> Locale:
        try:
            return self.locales[0]
        except IndexError:
            log.error("no locales specified")
            return None

    def install(self, bundle_dir: Path, source_dir: Path):
        for locale in self.locales:
            locale.install(bundle_dir, source_dir)

        if locale := self.main_locale:
            Plist(source_path=None).CFBundleLocalizations = [
                p.parent.parent.name
                for p in (
                    bundle_dir / "Contents" / "Resources" / "share" / "locale"
                ).rglob(locale.name)
            ]



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import List

from pydantic_xml import BaseXmlModel, element

from .framework import Framework

log = logging.getLogger("framework")


class Frameworks(BaseXmlModel):
    frameworks: List[Framework] = element(tag="framework")

    def install(self, bundle_dir: Path, source_dir: Path):
        for framework in self.frameworks:
            framework.install(bundle_dir, source_dir)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import Optional
from shutil import copy, copytree

from pydantic_xml import BaseXmlModel, attr

log = logging.getLogger("resource")


class Resource(BaseXmlModel):
    target_path: Optional[str] = attr(default=None)
    chmod: Optional[str] = attr(default=None)
    source_path: str

    @classmethod
    def path_relative_to(cls, path: Path, part: str) -> str:
        try:
            index = path.parts.index(part)
            return "/".join(path.parts[index:])
        except ValueError:
            return path

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Resources"

        for source_path in (source_dir / Path(self.source_path).parent).glob(
            Path(self.source_path).name
        ):
            if source_path.exists():
                if self.target_path:
                    target_path = target_dir / self.target_path
                else:
                    # source_path
                    #     a fully expanded path from self.source_path
                    # self.source_path
                    #     a path relative to source_dir, can contain globs
                    #
                    # Example:                   +----------------------------------+
                    #                            |                                  |
                    #                            v                                  |
                    #   self.source_path   =   share/glib-2.0/*.txt                 |
                    #   source_path        =   /some/path/share/glib-2.0/foo.txt    |
                    #                                       ^                       |
                    #                                       |                       |
                    #                                       +-----------------------+
                    #                           This is where we cut off source_path.
                    target_path = target_dir / self.path_relative_to(
                        source_path, Path(self.source_path).parts[0]
                    )
                if target_path.exists():
                    log.debug(f"will not overwrite {target_path}")
                else:
                    if not target_path.parent.exists():
                        # for subdirectories
                        target_path.parent.mkdir(parents=True)

                    log.debug(f"copy {source_path} to {target_path}")
                    if source_path.is_dir():
                        copytree(source_path, target_path, symlinks=True)
                    else:
                        copy(source_path, target_path)
                        if self.chmod:
                            target_path.chmod(int(self.chmod, 8))
            else:
                log.error(f"cannot locate {self.source_path}")



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path

from pydantic_xml import BaseXmlModel

from abcreate.bundle.library import Library

log = logging.getLogger("gtk")


class Gtk4(BaseXmlModel):
    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Frameworks"

        library = Library(source_path="libgtk-4.1.dylib")
        library.install(bundle_dir, source_dir)

        for source_path in Path(
            source_dir / "lib" / "gtk-4.0" / "4.0.0" / "printbackends"
        ).glob("*.so"):
            library = Library(source_path=source_path.as_posix())
            # Why flatten? We need to get rid of the subdirectories as e.g.
            # "4.0.0" in a path does not pass validation when signing.
            library.install(bundle_dir, source_dir, flatten=True)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import Optional
from shutil import copy

from pydantic_xml import BaseXmlModel, attr

from abcreate.util import LinkedObject
from .library import Library

log = logging.getLogger("executable")


class Executable(BaseXmlModel):
    name: Optional[str] = attr(default=None)
    source_path: str

    @property
    def target_name(self) -> str:
        if self.name:
            return self.name
        else:
            return Path(self.source_path).name

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "MacOS"
        if not target_dir.exists():
            target_dir.mkdir(parents=True)

        if (source_path := source_dir / "bin" / self.source_path).exists():
            target_path = target_dir / self.target_name
            if target_path.exists():
                log.error(f"will not overwrite {target_path}")
            else:
                log.debug(f"copy {source_path} to {target_path}")
                copy(source_path, target_path)

                # pull in dependencies
                lo = LinkedObject(source_path)
                for path in lo.flattened_dependency_tree(exclude_system=True):
                    library = Library(source_path=path.as_posix())
                    if library.is_framework:
                        # frameworks are taken care of separately
                        log.debug(
                            f"intentionally skipping framework library {library.source_path}"
                        )
                        pass
                    else:
                        library.install(bundle_dir, source_dir)

                # adjust install names: top level...
                frameworks_dir = bundle_dir / "Contents" / "Frameworks"
                lo = LinkedObject(target_path)
                lo.change_dependent_install_names(
                    "@executable_path/../Frameworks",
                    frameworks_dir.as_posix(),
                )
                # ...and one nesting level
                for sub_dir in filter(Path.is_dir, frameworks_dir.iterdir()):
                    lo.change_dependent_install_names(
                        f"@executable_path/../Frameworks/{sub_dir.name}",
                        sub_dir.as_posix(),
                    )
        else:
            log.error(f"cannot locate {self.source_path}")



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from pathlib import Path
import logging
from importlib.resources import files
from shutil import copy
from enum import Enum
from typing import Optional, ClassVar
import plistlib

from pydantic_xml import BaseXmlModel

log = logging.getLogger("plist")

INFO_PLIST = files("abcreate.bundle") / "Info.plist"


class Plist(BaseXmlModel):
    source_path: Optional[str] = None

    target_path: ClassVar[Path] = None

    class Key(Enum):
        CFBUNDLEEXECUTABLE = "CFBundleExecutable"
        CFBUNDLEICONFILE = "CFBundleIconFile"
        CFBUNDLELOCALIZATIONS = "CFBundleLocalizations"

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents"
        Plist.target_path = target_dir / "Info.plist"
        if Plist.target_path.exists():
            log.debug(f"already installed {Plist.target_path}")
        else:
            target_dir.mkdir(parents=True, exist_ok=True)
            if self.source_path:
                source_path = source_dir / self.source_path
            else:
                source_path = INFO_PLIST

            log.debug(f"copy {source_path} to {Plist.target_path}")
            copy(source_path, Plist.target_path)

    def _write(self, key: str, value: str):
        if plist := self._read_all():
            plist[key] = value
            with open(Plist.target_path, "wb") as file:
                plistlib.dump(plist, file)

    def _read_all(self):
        if Plist.target_path and Plist.target_path.exists():
            with open(Plist.target_path, "rb") as file:
                return plistlib.load(file)
        else:
            log.error("Info.plist hasn't been installed yet")
            return None

    def _read(self, key: str) -> str:
        if plist := self._read_all():
            return plist[key]
        else:
            return str()

    @property
    def CFBundleExecutable(self) -> str:
        return self._read(self.Key.CFBUNDLEEXECUTABLE.value)

    @CFBundleExecutable.setter
    def CFBundleExecutable(self, value):
        self._write(self.Key.CFBUNDLEEXECUTABLE.value, value)

    @property
    def CFBundleIconFile(self) -> str:
        return self._read(self.Key.CFBUNDLEICONFILE.value)

    @CFBundleIconFile.setter
    def CFBundleIconFile(self, value):
        self._write(self.Key.CFBUNDLEICONFILE.value, value)

    @property
    def CFBundleLocalizations(self):
        return self._read(self.Key.CFBUNDLELOCALIZATIONS.value)

    @CFBundleLocalizations.setter
    def CFBundleLocalizations(self, value):
        self._write(self.Key.CFBUNDLELOCALIZATIONS.value, value)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import List

from pydantic_xml import BaseXmlModel, element

from .executable import Executable
from .plist import Plist

log = logging.getLogger("executable")


class Executables(BaseXmlModel):
    executables: List[Executable] = element(tag="executable")

    @property
    def main_executable(self) -> Executable:
        try:
            return self.executables[0]
        except IndexError:
            log.critical("no executables specified")
            return None

    def install(self, bundle_dir: Path, source_dir: Path):
        for executable in self.executables:
            executable.install(bundle_dir, source_dir)

        Plist(source_path=None).CFBundleExecutable = self.main_executable.target_name



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
import re

from pydantic_xml import BaseXmlModel

from abcreate.bundle.library import Library

log = logging.getLogger("gtk")


class Gtk3(BaseXmlModel):
    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Frameworks"

        library = Library(source_path="libgtk-3.0.dylib")
        library.install(bundle_dir, source_dir)

        for source_path in Path(
            source_dir / "lib" / "gtk-3.0" / "3.0.0" / "immodules"
        ).glob("*.so"):
            library = Library(source_path=source_path.as_posix())
            # Why flatten? We need to get rid of the subdirectories as e.g.
            # "3.0.0" in a path does not pass validation when signing.
            library.install(bundle_dir, source_dir, flatten=True)

        for source_path in Path(
            source_dir / "lib" / "gtk-3.0" / "3.0.0" / "printbackends"
        ).glob("*.so"):
            library = Library(source_path=source_path.as_posix())
            # Why flatten? We need to get rid of the subdirectories as e.g.
            # "3.0.0" in a path does not pass validation when signing.
            library.install(bundle_dir, source_dir, flatten=True)

        target_dir = bundle_dir / "Contents" / "Resources"
        source_path = Path(source_dir / "lib" / "gtk-3.0" / "3.0.0" / "immodules.cache")
        immodules_cache = source_path.read_text()
        # Since we're breaking up the original structure, best place for
        # loaders.cache is etc as it is a configuration file after all.
        target_path = target_dir / "etc" / "immodules.cache"
        target_path.parent.mkdir(parents=True, exist_ok=True)
        with open(target_path, "wt") as file:
            for line in immodules_cache.splitlines(keepends=True):
                if match := re.match('".+(im-.+\.so)"', line):
                    file.write(f'"@executable_path/../Frameworks/{match.group(1)}"\n')
                else:
                    file.write(line)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from enum import Enum
from pathlib import Path
from typing import List, Set
import subprocess
import shlex
import re
import logging

log = logging.getLogger("linkedobj")


class LinkedObject:
    class SystemLinkPath(Enum):
        SYSTEM = "/System"
        USR = "/usr"

    class RelativeLinkPath(Enum):
        EXECUTABLE_PATH = "@executable_path"
        LOADER_PATH = "@loader_path"
        RPATH = "@rpath"

    resolved_rpaths: Set[Path] = set()

    @classmethod
    def is_framework(cls, path: Path) -> bool:
        return ".framework/" in path.as_posix()

    @classmethod
    def is_relative_path(cls, path: Path, relative_to=str()) -> bool:
        if relative_to:
            return path.is_relative_to(relative_to)
        else:
            return (
                path.parts[0] in [item.value for item in LinkedObject.RelativeLinkPath]
                or not path.parent.is_absolute()
            )

    @classmethod
    def is_system_path(cls, path: Path) -> bool:
        # Why 0:2? the leading slash counts as part
        return "".join(path.parts[0:2]) in [item.value for item in cls.SystemLinkPath]

    def __init__(self, path: Path):
        self.path = path
        # TODO: This depends on being populated before we encounter the first lib
        # that uses rpath.
        if not LinkedObject.is_relative_path(self.path):
            self._resolve_rpaths()

    def _make_absolute(self, path: Path) -> Path:
        if LinkedObject.is_relative_path(path) and not LinkedObject.is_framework(path):
            if LinkedObject.is_relative_path(
                path, LinkedObject.RelativeLinkPath.RPATH.value
            ):
                for rpath in self.resolved_rpaths:
                    potential_path = rpath / "/".join(path.parts[1:])
                    if potential_path.is_file():
                        return potential_path
                log.error(f"failed to resolve rpath for {path}")
                return path
            else:
                match self.path.parent.name:
                    case "bin":
                        return self.path.parent.parent / "lib" / path.name
                    case _:
                        return self.path.parent / path.name
        else:
            return path

    def _otool(self, args: str) -> List[str]:
        """Run ``otool`` and return its output.

        Parameters
        ----------
        args : str
            The arguments for the ``otool`` command. This includes options
            and filename.

        Returns
        -------
        List[str]
            The commmand's stdout as list of strings.
        """
        try:
            sp = subprocess.run(
                shlex.split(f"/usr/bin/otool {args} {self.path}"),
                capture_output=True,
                encoding="utf-8",
            )
            sp.check_returncode()
            return sp.stdout.splitlines()
        except subprocess.CalledProcessError:
            log.error(f"otool {args} failed for {self.path}")
            return list()

    def _install_name_tool(self, args: str) -> list:
        try:
            sp = subprocess.run(
                shlex.split(f"/usr/bin/install_name_tool {args} {self.path}"),
                capture_output=True,
                encoding="utf-8",
            )
            sp.check_returncode()
            return sp.stdout.splitlines()
        except subprocess.CalledProcessError:
            log.error(f"install_name_tool {args} failed for {self.path}")
            return list()

    @property
    def install_name(self) -> str:
        result = self._otool("-D")
        return result[1] if len(result) == 2 else ""

    @install_name.setter
    def install_name(self, install_name: str):
        self._install_name_tool(f"-id {install_name}")

    @property
    def rpaths(self) -> List[Path]:
        result = list()
        line_iter = iter(self._otool("-l"))
        for line in line_iter:
            if re.match("\s+cmd LC_RPATH", line):
                next(line_iter)
                if match := re.match("\s+path (.+) \(offset.+", next(line_iter)):
                    result.append(Path(match.group(1)))
        return result

    def _resolve_rpaths(self) -> None:
        for rpath in self.rpaths:
            if rpath.parts[0] in (LinkedObject.RelativeLinkPath.LOADER_PATH.value,):
                LinkedObject.resolved_rpaths.add(
                    (self.path.parent / "/".join(rpath.parts[1:])).resolve()
                )

    def add_rpath(self, rpath: str):
        self._install_name_tool(f"-add_rpath {rpath}")

    def change_one_dependant_install_name(self, install_name: str):
        if libs := [l for l in self.depends_on() if Path(install_name).name in l]:
            self._install_name_tool(f"-change {libs[0]} {install_name}")

    def change_dependent_install_names(self, install_name: str, lib_dir: str):
        libs_in_lib_dir = [
            l for l in Path(lib_dir).glob("*.dylib") if not l.is_symlink()
        ]

        for lib in self.depends_on():
            if Path(lib).name in [Path(l).name for l in libs_in_lib_dir]:
                self._install_name_tool(
                    f"-change {lib} {Path(install_name)/Path(lib).name}"
                )

    def clear_rpath(self):
        for rpath in self.rpath:
            self._install_name_tool(f"-delete_rpath {rpath}")

    def depends_on(self, exclude_system: bool = False) -> List[Path]:
        result = list()

        if LinkedObject.is_framework(self.path):
            log.debug(f"intentionally skipping framework library {self.path}")
        else:
            skip_lines = 2 if self.install_name else 1
            for line in self._otool("-L")[skip_lines:]:
                # This matches only dylibs:
                # match = re.match("\t(.+\.dylib)", line)
                # This will match everything:
                if match := re.match("\t(.+) \(compatibility", line):
                    library = Path(match.group(1))
                    if exclude_system:
                        if not LinkedObject.is_system_path(library):
                            result.append(library)
                    else:
                        result.append(library)
        return result

    def flattened_dependency_tree(
        self, exclude_system: bool = False, _dependencies=list()
    ) -> List[Path]:
        for library in self.depends_on(exclude_system):
            library = self._make_absolute(library)
            if library not in _dependencies:
                _dependencies.append(library)
                for l in LinkedObject(library).flattened_dependency_tree(
                    exclude_system,
                    _dependencies,
                ):
                    if l not in _dependencies:
                        _dependencies.append(l)
        return _dependencies



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from shutil import copy

from pydantic_xml import BaseXmlModel

log = logging.getLogger("locale")


class Locale(BaseXmlModel):
    name: str

    @classmethod
    def path_relative_to(cls, path: Path, part: str) -> str:
        try:
            index = path.parts.index(part)
            return "/".join(path.parts[index + 1 :])
        except ValueError:
            return path

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Resources" / "share" / "locale"

        for source_path in Path(source_dir / "share" / "locale").rglob(self.name):
            target_path = target_dir / self.path_relative_to(source_path, "locale")
            if target_path.exists():
                log.debug(f"will not overwrite {target_path}")
            else:
                log.debug(f"copy {source_path} to {target_path}")
                target_path.parent.mkdir(parents=True, exist_ok=True)
                copy(source_path, target_path)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

from .gdkpixbuf import GdkPixbuf
from .gir import Gir
from .gtk3 import Gtk3
from .gtk4 import Gtk4



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from shutil import copy

from pydantic_xml import BaseXmlModel

from .plist import Plist

log = logging.getLogger("icon")


class Icon(BaseXmlModel):
    source_path: str

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Resources"
        target_dir.mkdir(parents=True, exist_ok=True)

        source_path = source_dir / self.source_path
        target_path = target_dir / source_path.name

        if target_path.exists():
            log.error(f"will not overwrite {target_path}")
        else:
            log.debug(f"copy {source_path} to {target_path}")
            copy(source_path, target_path)

            Plist(source_path=None).CFBundleIconFile = target_path.name



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from lxml import etree
import shlex
import subprocess
from tempfile import TemporaryDirectory
from typing import Optional

from pydantic_xml import BaseXmlModel, attr

from abcreate.bundle.library import Library

log = logging.getLogger("gir")


class Gir(BaseXmlModel):
    lib_dir: Optional[str] = attr(default="@executable_path/../Frameworks")
    command: Optional[str] = attr(default="g-ir-compiler")

    def _compile(self, source_path: Path, target_path: Path):
        log.debug(f"compiling {target_path}")
        try:
            subprocess.run(
                [
                    *shlex.split(self.command),
                    "-o",
                    target_path,
                    source_path,
                ]
            ).check_returncode()
        except FileNotFoundError:
            log.error(f"command not found: {self.command}")
        except subprocess.CalledProcessError as e:
            log.error(f"typelib compilation failed\n{e}")

    def install(self, bundle_dir: Path, source_dir: Path):
        target_dir = bundle_dir / "Contents" / "Resources" / "lib" / "girepository-1.0"
        target_dir.mkdir(parents=True, exist_ok=True)

        library = Library(source_path="libgirepository-1.0.1.dylib")
        library.install(bundle_dir, source_dir)

        for source_path in Path(source_dir / "share" / "gir-1.0").glob("*.gir"):
            target_path = target_dir / source_path.with_suffix(".typelib").name

            tree = etree.parse(source_path)
            nsmap = {
                "core": "http://www.gtk.org/introspection/core/1.0",
                "c": "http://www.gtk.org/introspection/c/1.0",
                "glib": "http://www.gtk.org/introspection/glib/1.0",
            }
            try:
                element = tree.xpath(
                    "//core:repository/core:namespace", namespaces=nsmap
                )[0]
                libraries = element.attrib["shared-library"].split(",")
                element.attrib["shared-library"] = ""

                for library in libraries:
                    if len(element.attrib["shared-library"]):
                        element.attrib["shared-library"] += ","
                    element.attrib["shared-library"] += (
                        Path(self.lib_dir) / Path(library).name
                    ).as_posix()

                with TemporaryDirectory() as temp_dir:
                    gir_file = Path(temp_dir) / source_path.name
                    tree.write(gir_file, pretty_print=True)
                    self._compile(gir_file, target_path)
            except KeyError:
                log.debug(f"no shared-library in {target_path}")
                self._compile(source_path, target_path)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
import os
from pathlib import Path

from pydantic_xml import BaseXmlModel

log = logging.getLogger("symlink")


class Symlink(BaseXmlModel):
    name: str

    def install(self, bundle_dir: Path):
        target_dir = bundle_dir / "Contents" / "MacOS"

        source_path = Path(self.name)
        target_path = target_dir / source_path.name

        if target_path.exists():
            log.debug(f"will not overwrite {target_path}")
        else:
            log.debug(f"symlinking {source_path} to {target_path}")
            target_path.parent.mkdir(parents=True, exist_ok=True)
            os.symlink(src=source_path, dst=target_path)



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from shutil import copy

from pydantic_xml import BaseXmlModel
from pydantic import field_validator

from abcreate.util import LinkedObject

log = logging.getLogger("library")


class Library(BaseXmlModel):
    source_path: str

    @field_validator("source_path")
    def ensure_relative_path(cls, value: str) -> str:
        path = Path(value)
        if path.is_absolute():
            return cls.path_relative_to(path, "lib").as_posix()
        elif LinkedObject.is_relative_path(path):
            return f"{path.name}"
        else:
            return value

    @classmethod
    def path_relative_to(cls, path: Path, part: str) -> Path:
        try:
            index = path.parts.index(part)
            return Path(*list((path.parts[index + 1 :])))
        except ValueError:
            return path

    @property
    def is_framework(self) -> bool:
        return ".framework/" in self.source_path

    def install(self, bundle_dir: Path, source_dir: Path, flatten: bool = False):
        target_dir = bundle_dir / "Contents" / "Frameworks"
        if not target_dir.exists():
            log.debug(f"creating {target_dir}")
            target_dir.mkdir(parents=True)

        for source_path in (source_dir / "lib" / Path(self.source_path).parent).glob(
            Path(self.source_path).name
        ):
            if source_path.exists():
                if flatten:
                    target_path = target_dir / source_path.name
                else:
                    target_path = target_dir / self.path_relative_to(source_path, "lib")
                if target_path.exists():
                    log.debug(f"will not overwrite {target_path}")
                else:
                    if not target_path.parent.exists():
                        # for subdirectories in the libraries directory
                        target_path.parent.mkdir(parents=True)

                    log.debug(f"copy {source_path} to {target_path}")
                    copy(source_path, target_path)

                    # pull in dependencies
                    lo = LinkedObject(source_path)
                    for path in lo.flattened_dependency_tree(exclude_system=True):
                        library = Library(source_path=path.as_posix())
                        if library.is_framework:
                            # frameworks are taken care of separately
                            log.debug(
                                f"intentionally skipping framework library {library.source_path}"
                            )
                        else:
                            library.install(bundle_dir, source_dir)

                    # adjust install names
                    lo = LinkedObject(target_path)
                    loader_path = Path("@loader_path")
                    if not flatten:
                        # take care of nested directory structure
                        for _ in range(
                            len(self.path_relative_to(source_path, "lib").parts) - 1
                        ):
                            loader_path /= ".."
                    lo.change_dependent_install_names(
                        loader_path.as_posix(), target_dir.as_posix()
                    )
            else:
                log.error(f"cannot locate {self.source_path}")



# SPDX-FileCopyrightText: 2025 René de Hesselle <dehesselle@web.de>
#
# SPDX-License-Identifier: GPL-2.0-or-later

import logging
from pathlib import Path
from typing import List

from pydantic_xml import BaseXmlModel, element

from .symlink import Symlink

log = logging.getLogger("symlink")


class Symlinks(BaseXmlModel):
    symlinks: List[Symlink] = element(tag="symlink")

    def install(self, bundle_dir: Path):
        for symlink in self.symlinks:
            symlink.install(bundle_dir)
